@@ -620,7 +620,8 @@ int bgzf_check_EOF(BGZF *fp)
 	off_t offset;
 	offset = _bgzf_tell((_bgzf_file_t)fp->fp);
 	if (_bgzf_seek(fp->fp, -28, SEEK_END) < 0) return 0;
-	_bgzf_read(fp->fp, buf, 28);
+	int ret = _bgzf_read(fp->fp, buf, 28);
+        UNUSED(ret);
 	_bgzf_seek(fp->fp, offset, SEEK_SET);
 	return (memcmp(magic, buf, 28) == 0)? 1 : 0;
 }
